#
# Cookbook Name:: normal_site
# Recipe:: default
#
# Copyright sgalinski Internet Services
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

base_hostname = node['normal_site']['hostname'] || node['ubuntu_base']['hostname']

##########################
### Add hostname entry ###
##########################

hostsfile_entry node['ubuntu_base']['ip_address'] do
	hostname base_hostname
	action :append
end

hostsfile_entry node['ubuntu_base']['ip_address'] do
	hostname node['normal_site']['server_aliases']
	action :append
end

node['normal_site']['create_additional_hostentries'].each do |hostentry|
	hostsfile_entry hostentry['ip_address'] do
		hostname hostentry['hostname']
		action :append
	end
end

##############################
### Add additional folders ###
##############################

node['normal_site']['create_additional_folders'].each do |folder|
	execute 'create additional folder' do
		command "mkdir -p #{folder['folder']}"
		action :run
	end
end

###################################
### Install additional packages ###
###################################

if node['platform_version'] == '22.04'
	apt_repository('php') do
		uri 'http://ppa.launchpad.net/ondrej/php/ubuntu'
		deb_src true
		trusted true
		keyserver 'keyserver.ubuntu.com'
		key 'E5267A6C'
		distribution 'jammy'
		components [:main]
		arch 'amd64'
		action :add
	end

elsif node['platform_version'] == '20.04'
	apt_repository('php') do
		uri 'http://ppa.launchpad.net/ondrej/php/ubuntu'
		deb_src true
		trusted true
		keyserver 'keyserver.ubuntu.com'
		key 'E5267A6C'
		distribution 'focal'
		components [:main]
		arch 'amd64'
		action :add
	end

elsif node['platform_version'] == '16.04'
	apt_repository('apache2') do
		uri 'http://ppa.launchpad.net/ondrej/apache2/ubuntu'
		deb_src true
		trusted true
		keyserver 'keyserver.ubuntu.com'
		key 'E5267A6C'
		distribution 'xenial'
		components [:main]
		arch 'amd64'
		action :add
	end

	apt_repository('php7.1') do
		uri 'http://ppa.launchpad.net/ondrej/php/ubuntu'
		deb_src true
		trusted true
		keyserver 'keyserver.ubuntu.com'
		key 'E5267A6C'
		distribution 'xenial'
		components [:main]
		arch 'amd64'
		action :add
	end
end

execute 'apt -y update'
package 'graphicsmagick'
package 'apache2'

# Directly execute apt to improve performance and prevent multiple fpm reloads
if node['ubuntu_base']['php_version'] == '7.0'
	bash 'Install PHP' do
		code 'apt install -y php7.0-fpm php7.0-cli php7.0-curl php7.0-intl php7.0-gd php7.0-mcrypt php7.0-mysql php7.0-xml php7.0-json php7.0-mbstring php7.0-soap php7.0-zip php7.0-imagick'
		action :run
	end
elsif node['ubuntu_base']['php_version'] == '7.1'
	bash 'Install PHP' do
		code 'apt install -y php7.1-fpm php7.1-cli php7.1-curl php7.1-intl php7.1-gd php7.1-mcrypt php7.1-mysql php7.1-xml php7.1-json php7.1-mbstring php7.1-soap php7.1-zip php7.1-imagick'
		action :run
	end
elsif node['ubuntu_base']['php_version'] == '7.2'
	bash 'Install PHP' do
		code 'apt install -y php7.2-fpm php7.2-cli php7.2-curl php7.2-intl php7.2-gd php7.2-mysql php7.2-xml php7.2-json php7.2-mbstring php7.2-soap php7.2-zip php7.2-imagick'
		action :run
	end
elsif node['ubuntu_base']['php_version'] == '7.3'
	bash 'Install PHP' do
		code 'apt install -y php7.3-fpm php7.3-cli php7.3-curl php7.3-intl php7.3-gd php7.3-mysql php7.3-xml php7.3-json php7.3-mbstring php7.3-soap php7.3-zip php7.3-imagick'
		action :run
	end
elsif node['ubuntu_base']['php_version'] == '7.4'
	bash 'Install PHP' do
		code 'apt install -y php7.4-fpm php7.4-cli php7.4-curl php7.4-intl php7.4-gd php7.4-mysql php7.4-xml php7.4-json php7.4-mbstring php7.4-soap php7.4-zip php7.4-imagick'
		action :run
	end
elsif node['ubuntu_base']['php_version'] == '8.1'
	bash 'Install PHP' do
		code 'apt install -y php8.1-fpm php8.1-cli php8.1-curl php8.1-intl php8.1-gd php8.1-mysql php8.1-xml php8.1-mbstring php8.1-soap php8.1-zip php8.1-imagick'
		action :run
	end
else
	bash 'Install PHP' do
		code 'apt install -y php8.2-fpm php8.2-cli php8.2-curl php8.2-intl php8.2-gd php8.2-mysql php8.2-xml php8.2-mbstring php8.2-soap php8.2-zip php8.2-imagick'
		action :run
	end
end

# enable Apache modules
%w{expires deflate rewrite headers http2 ssl proxy_fcgi mpm_event}.each do |module_name|
	execute 'a2enmod ' + module_name
end

####################################
### Modify PHP FPM Configuration ###
####################################

template "/etc/php/#{node['ubuntu_base']['php_version']}/fpm/pool.d/www.conf" do
	source 'www.conf.erb'
	owner 'root'
	group 'root'
	variables(
	  {
		:php_version => node['ubuntu_base']['php_version']
	  }
	)
end

chef_sleep 'Wait some seconds to prevent a too fast repeating FPM reload crash' do
	seconds 1
end

service "php#{node['ubuntu_base']['php_version']}-fpm" do
	action :restart
end

################################
### Modify PHP Configuration ###
################################

replace_or_add 'Increase time limit' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/fpm/php.ini'
	pattern 'max_execution_time =.*'
	line 'max_execution_time = 240'
end

replace_or_add 'Increase time limit - CLI' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/cli/php.ini'
	pattern 'max_execution_time =.*'
	line 'max_execution_time = 240'
end

replace_or_add 'Increase memory limit' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/fpm/php.ini'
	pattern 'memory_limit =.*'
	line 'memory_limit = 512M'
end

replace_or_add 'Increase upload size limit' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/fpm/php.ini'
	pattern 'upload_max_filesize =.*'
	line 'upload_max_filesize = 100M'
end

replace_or_add 'Increase upload size limit - CLI' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/cli/php.ini'
	pattern 'upload_max_filesize =.*'
	line 'upload_max_filesize = 100M'
end

replace_or_add 'Increase post_max_size limit' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/fpm/php.ini'
	pattern 'post_max_size =.*'
	line 'post_max_size = 112M'
end

replace_or_add 'Increase post_max_size limit - CLI' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/cli/php.ini'
	pattern 'post_max_size =.*'
	line 'post_max_size = 112M'
end

replace_or_add 'Set date.timezone' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/fpm/php.ini'
	pattern 'date.timezone =.*'
	line 'date.timezone = Europe/Berlin'
end

replace_or_add 'Set date.timezone - CLI' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/cli/php.ini'
	pattern 'date.timezone =.*'
	line 'date.timezone = Europe/Berlin'
end

replace_or_add 'Set max_input_vars' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/fpm/php.ini'
	pattern '.*max_input_vars =.*'
	line 'max_input_vars = 1500'
end

replace_or_add 'Set max_input_vars - CLI' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/cli/php.ini'
	pattern '.*max_input_vars =.*'
	line 'max_input_vars = 1500'
end

execute 'mkdir -p /etc/apache2/ssl'

replace_or_add 'Set curl ca certificate' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/fpm/php.ini'
	pattern '.*curl.cainfo =.*'
	line 'curl.cainfo = /etc/apache2/ssl/rootCA.pem'
end

replace_or_add 'Set curl ca certificate - CLI' do
	path '/etc/php/' + node['ubuntu_base']['php_version'] + '/cli/php.ini'
	pattern '.*curl.cainfo =.*'
	line 'curl.cainfo = /etc/apache2/ssl/rootCA.pem'
end

#######################
### Install ioncube ###
#######################

if node['normal_site']['install_ioncube']
	directory '/usr/local/ioncube' do
		mode 0755
		action :create
		not_if { ::Dir.exists?('/usr/local/ioncube') }
	end

	remote_file '/usr/local/ioncube/ioncube_loaders_lin_x86-64.tar.gz' do
		source 'https://downloads.ioncube.com/loader_downloads/ioncube_loaders_lin_x86-64.tar.gz'
		mode 0644
		action :create_if_missing
		not_if 'test -f /usr/local/ioncube/ioncube_loaders_lin_x86-64.tar.gz'
	end

	execute 'tar --no-same-owner --strip-components=1 -zxvf ioncube_loaders_lin_x86-64.tar.gz' do
		cwd '/usr/local/ioncube'
	end

	template '/etc/php/' + node['ubuntu_base']['php_version'] + '/mods-available/ioncube.ini' do
		source 'ioncube.ini.erb'
		variables(
		  {
			:php_version => node['ubuntu_base']['php_version']
		  }
		)
	end

	execute 'ioncube php module enable' do
		command '/usr/sbin/phpenmod ioncube && cd /etc/php/' + node['ubuntu_base']['php_version'] +
		  '/fpm/conf.d && mv 20-ioncube.ini 1-ioncube.ini && cd /etc/php/' +
		  node['ubuntu_base']['php_version'] + '/cli/conf.d && mv 20-ioncube.ini 1-ioncube.ini'
	end
end

######################################
### Install additional PHP modules ###
######################################

node['normal_site']['additional_php_modules'].each do |additionalPhpModule|
	package 'php' + node['ubuntu_base']['php_version'] + '-' + additionalPhpModule['name'] do
		action 'upgrade'
	end

	execute "phpenmod " + additionalPhpModule['name']

	service "php#{node['ubuntu_base']['php_version']}-fpm" do
		action :restart
	end
end

################################
### Setup Apache Environment ###
################################

# create ssl certificate
# Instructions are from here: https://stackoverflow.com/questions/7580508/getting-chrome-to-accept-self-signed-localhost-certificate/43666288#43666288
template '/etc/apache2/ssl/v3.ext' do
	source 'v3.ext'
end

template '/etc/apache2/ssl/create_certificate_for_domain.sh' do
	source 'create_certificate_for_domain.sh'
end

# not needed (globally used, but maybe someone else needs this)
# template 'create_root_cert_and_key.sh' do
# 	source 'create_root_cert_and_key.sh'
# end

template '/etc/apache2/ssl/rootCA.key' do
	source 'rootCA.key'
end

template '/etc/apache2/ssl/rootCA.pem' do
	source 'rootCA.pem'
end

# Serial file (don't use that as you can't create more than one certificate if you add this!)
# template '/etc/apache2/ssl/rootCA.srl' do
# 	source 'rootCA.srl'
# end

# concatenate the additional optional domains in order to avoid missing index exceptions
additional_domains = ''
if node.include?('webgrind') && node['webgrind'].include?('hostname') && node['webgrind']['hostname']
	additional_domains = additional_domains + ' ' + node['webgrind']['hostname']
else
	additional_domains = additional_domains + ' webgrind.' + base_hostname
end

if node.include?('mailhog') && node['mailhog'].include?('hostname') && node['mailhog']['hostname']
	additional_domains = additional_domains + ' ' + node['mailhog']['hostname']
else
	additional_domains = additional_domains + ' mailhog.' + base_hostname
end

if node.include?('mailpit') && node['mailpit'].include?('hostname') && node['mailpit']['hostname']
	additional_domains = additional_domains + ' ' + node['mailpit']['hostname']
else
	additional_domains = additional_domains + ' mailpit.' + base_hostname
end

# Add the additional vhosts to the additional_domains...
node['normal_site']['create_additional_vhosts'].each do |vhost|
	if vhost['hostname']
		additional_domains = additional_domains + ' ' + vhost['hostname']
	end
end

bash 'Create Certificate' do
	cwd '/etc/apache2/ssl/'
	code <<-EOF
		chmod 755 create_certificate_for_domain.sh
		./create_certificate_for_domain.sh #{base_hostname} #{node['normal_site']['server_aliases']}#{additional_domains}
	EOF
	action :run
end

# create vHost directory
directory "#{node['normal_site']['webroot']}/#{base_hostname}" do
	mode '0755'
	action :create
end

# create web app with an own virtual host
template '/etc/apache2/sites-available/' + base_hostname + '.conf' do
	source 'vHost.conf.erb'
	mode '0644'
	owner 'root'
	group 'root'

	variables(
	  {
		:docroot => "#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/",
		:server_name => base_hostname,
		:base_hostname => base_hostname,
		:server_aliases => node['normal_site']['server_aliases'],
		:php_version => node['ubuntu_base']['php_version']
	  }
	)
end

execute "a2ensite #{base_hostname}.conf"

# Do the same stuff from above for additional vhost entries...
node['normal_site']['create_additional_vhosts'].each do |vhost|
	if vhost['hostname']
		# create vHost directory
		directory "#{node['normal_site']['webroot']}/#{vhost['hostname']}" do
			mode '0755'
			action :create
		end

		# create web app with an own virtual host
		template '/etc/apache2/sites-available/' + vhost['hostname'] + '.conf' do
			source 'vHost.conf.erb'
			mode '0644'
			owner 'root'
			group 'root'

			variables(
			  {
				:docroot => "#{node['normal_site']['webroot']}/#{vhost['hostname']}/#{vhost['web_directory']}/",
				:server_name => vhost['hostname'],
				:base_hostname => base_hostname,
				:server_aliases => vhost['server_aliases'],
				:php_version => node['ubuntu_base']['php_version']
			  }
			)
		end

		execute "a2ensite #{vhost['hostname']}.conf"
	end
end

# Switch to vagrant as Apache main user
replace_or_add 'Change the apache group to the users main group' do
	path '/etc/apache2/envvars'
	pattern 'export APACHE_RUN_GROUP=.*'
	line "export APACHE_RUN_GROUP=vagrant"
end

replace_or_add 'Change the apache group to the users main group' do
	path '/etc/apache2/envvars'
	pattern 'export APACHE_RUN_USER=.*'
	line "export APACHE_RUN_USER=vagrant"
end

service 'apache2' do
	action :restart
end

#######################
### Provide Sources ###
#######################

file '/home/vagrant/ssh_wrapper.sh' do
	owner 'vagrant'
	group 'vagrant'
	mode '0755'
	content "#!/bin/sh\nexec /usr/bin/ssh -o \"StrictHostKeyChecking=no\" \"$@\""
end

node['normal_site']['sync_repositories'].each do |sync_repository|
	# don't fix this. This must look exactly like this! Foodcritic is not correct here!
	if sync_repository['hostname']
		ssh_known_hosts_entry sync_repository['hostname'] do
			port sync_repository['port']
		end
	end

	git "#{node['normal_site']['webroot']}/#{sync_repository['folder_name']}" do
		remote 'origin'
		repository sync_repository['repository']
		checkout_branch sync_repository['branch'] ? sync_repository['branch'] : "master"
		enable_checkout false
		revision sync_repository['branch'] ? sync_repository['branch'] : "master"
		timeout 100000
		action :checkout
		ssh_wrapper '/home/vagrant/ssh_wrapper.sh'
	end
end

###################
### Setup TYPO3 ###
###################

bash 'Setup TYPO3' do
	cwd '/home/vagrant/'

	user 'vagrant'
	group 'vagrant'
	timeout 3000000
	only_if { node['normal_site']['download_typo3'] }

	code <<-EOF
		if [ ! -d #{node['normal_site']['webroot']}/typo3_src-#{node['normal_site']['typo3_version']} ]
		then
			wget http://get.typo3.org/#{node['normal_site']['typo3_version']} \
				-O typo3_src-#{node['normal_site']['typo3_version']}.tgz

			mkdir -p #{node['normal_site']['webroot']}/typo3_src-#{node['normal_site']['typo3_version']}
			tar -xzf typo3_src-#{node['normal_site']['typo3_version']}.tgz --strip-components=1 \
				-C #{node['normal_site']['webroot']}/typo3_src-#{node['normal_site']['typo3_version']};
		fi

		mkdir -p #{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}
		cd #{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}
		rm -f typo3_src
		ln -sf ../typo3_src-#{node['normal_site']['typo3_version']} typo3_src
		touch typo3conf/ENABLE_INSTALL_TOOL

		ln -sf typo3_src/typo3 typo3
		ln -sf typo3_src/index.php index.php
	EOF
	action :run
end

bash 'Setup TYPO3 (Composer)' do
	cwd #{node['normal_site']['webroot']}

	user 'vagrant'
	group 'vagrant'
	timeout 3000000
	only_if { node['normal_site']['setup_typo3_composer'] }

	code <<-EOF
		if [ ! -f #{node['normal_site']['webroot']}/#{base_hostname}/composer.json ]
		then
			cd #{node['normal_site']['webroot']}

			if ! command -v composer &> /dev/null
			then
				export COMPOSER_HOME=/home/vagrant/.composer
				php -r "copy('https://getcomposer.org/installer', 'composer-setup.php');"
				php composer-setup.php
				php -r "unlink('composer-setup.php');"
				sudo mv composer.phar /usr/local/bin/composer
			fi

			composer create-project typo3/cms-base-distribution:^#{node['normal_site']['typo3_version']} #{base_hostname}
			touch #{base_hostname}/#{node['normal_site']['web_directory']}/FIRST_INSTALL
		fi
	EOF
	action :run
end

######################
### Setup Shopware ###
######################

if node['normal_site']['shopware_version'][0] == '5'
	bash 'Setup Shopware 5x' do
		cwd '/home/vagrant/'

		user 'vagrant'
		group 'vagrant'
		timeout 3000000
		only_if { node['normal_site']['download_shopware'] }

		code <<-EOF
		if [ ! -d #{node['normal_site']['webroot']}/#{base_hostname}/config ]
		then
			wget https://github.com/shopware/shopware/archive/v#{node['normal_site']['shopware_version']}.zip \
				-O shopware.zip
			
			unzip shopware.zip -d #{node['normal_site']['webroot']}/#{base_hostname}/;
		fi
		EOF
		action :run
	end
else
	bash 'Setup Shopware 6x' do
		cwd '/home/vagrant/'

		user 'vagrant'
		group 'vagrant'
		timeout 3000000
		only_if { node['normal_site']['download_shopware'] }

		code <<-EOF
		if [ ! -f #{node['normal_site']['webroot']}/#{base_hostname}/composer.json ]
		then
			cd #{node['normal_site']['webroot']}

			if ! command -v composer &> /dev/null
			then
				export COMPOSER_HOME=/home/vagrant/.composer
				php -r "copy('https://getcomposer.org/installer', 'composer-setup.php');"
				php composer-setup.php
				php -r "unlink('composer-setup.php');"
				sudo mv composer.phar /usr/local/bin/composer
			fi

			composer create-project shopware/production:dev-flex temp v#{node['normal_site']['shopware_version']}

			shopt -s dotglob
			mv temp/* #{base_hostname}
			rm -rf temp

			touch #{base_hostname}/FIRST_INSTALL
		fi
		EOF
		action :run
	end
end

#########################
### Synchronize Files ###
#########################

node['normal_site']['sync_directories'].each do |sync_data|
	if sync_data['hostname']
		ssh_known_hosts_entry sync_data['hostname'] do
			port sync_data['port']
		end
	end

	bash "Synchronize #{sync_data['local']}" do
		user 'vagrant'
		group 'vagrant'
		timeout 3000000
		only_if {
			sync_data['sync'] or not ::File.exists?("#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/#{sync_data['local']}")
		}

		keep_non_existing_files = '--delete'
		if sync_data['keep_non_existing_files']
			keep_non_existing_files = ''
		end

		if sync_data['hostname']
			code <<-EOF
					rsync #{keep_non_existing_files} -auvz --progress -e "ssh -p #{sync_data['port']} -o StrictHostKeyChecking=no" \
						--no-o --no-g #{sync_data['user']}@#{sync_data['hostname']}:#{sync_data['remote']} \
						"#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/#{sync_data['local']}";
			EOF
		else
			code <<-EOF
					rsync #{keep_non_existing_files} -auvz --progress --no-o --no-g #{sync_data['remote']} \
					"#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/#{sync_data['local']}";
			EOF
		end

		action :run
	end
end

####################
### Create Links ###
####################

node['normal_site']['create_links'].each do |link_data|
	execute 'remove link target' do
		command "rm -f #{link_data['target']}"
		action :run
	end

	link link_data['source'] do
		to link_data['target']
	end
end

#################################################
### Provide TYPO3 site specific configuration ###
#################################################

if node['normal_site']['configure_typo3']
	directory "#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/typo3conf/" do
		mode '0755'
		recursive true
		action :create
	end

	directory "#{node['normal_site']['webroot']}/#{base_hostname}/config/system/" do
		mode '0755'
		recursive true
		action :create
	end

	template "#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/typo3conf/SiteConfiguration.php" do
		source 'typo3/SiteConfiguration.php.erb'
		mode '0660'
	end

	template "#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/typo3conf/AdditionalConfiguration.php" do
		source 'typo3/AdditionalConfiguration.php'
		mode '0660'
		action :create_if_missing
	end

	template "#{node['normal_site']['webroot']}/#{base_hostname}/config/system/environment.php" do
		source 'typo3/environment.php.erb'
		mode '0660'
	end

	template "#{node['normal_site']['webroot']}/#{base_hostname}/config/system/additional.php" do
		source 'typo3/additional.php'
		mode '0660'
		action :create_if_missing
	end

	template "#{node['normal_site']['webroot']}/#{base_hostname}/.env.local" do
		source 'typo3/env.local.erb'
		mode '0660'
	end
end

####################################################
### Provide Shopware site specific configuration ###
####################################################

if node['normal_site']['configure_shopware']
	if node['normal_site']['shopware_version'][0] == '5'
		file "#{node['normal_site']['webroot']}/#{base_hostname}/config.php" do
			action :delete
		end

		template "#{node['normal_site']['webroot']}/#{base_hostname}/config.php" do
			source 'shopware/config.php.erb'
			mode '0755'
		end
	else
		file "#{node['normal_site']['webroot']}/#{base_hostname}/.env.local" do
			action :delete
		end

		template "#{node['normal_site']['webroot']}/#{base_hostname}/.env.local" do
			source 'shopware/.env.local.erb'
			mode '0755'
		end
	end
end

#####################################################
### Provide Wordpress site specific configuration ###
#####################################################

if node['normal_site']['configure_wordpress']
	directory "#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/" do
		mode '0755'
		action :create
	end

	template "#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/wp-config.php" do
		source 'wordpress/wp-config.php.erb'
		mode '0660'
		variables(
		  :params => {
			:hostname => base_hostname,
			:table_prefix => node['normal_site']['wordpress_table_prefix']
		  }
		)
	end

	template "#{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/.htaccess" do
		source 'wordpress/htaccess.erb'
		mode '0660'
	end
end

###################
### Setup MySQL ###
###################

package 'mysql-server-' + node['normal_site']['mysql_version']

template '/etc/mysql/mysql.cnf' do
	source "my-#{node['normal_site']['mysql_version']}.cnf.erb"
	owner 'root'
	group 'root'
end

service 'mysql' do
	action :restart
end

########################
### Create Databases ###
########################

node['normal_site']['sync_databases'].each do |database_data|
	bash 'Setup/Import database and user' do
		code <<-EOF
			mysql -u root -proot -e "drop database if exists \\\`#{database_data['database_name']}\\\`;"
			mysql -u root -proot -e "create database if not exists \\\`#{database_data['database_name']}\\\`;"
			mysql -u root -proot -e "DROP USER '#{database_data['database_user']}'@'127.0.0.1';"
			mysql -u root -proot -e "create user '#{database_data['database_user']}'@'127.0.0.1' IDENTIFIED WITH mysql_native_password BY '#{database_data['database_password']}';"
			mysql -u root -proot -e "grant usage on \\\`#{database_data['database_name']}\\\`.* to '#{database_data['database_user']}'@'127.0.0.1' IDENTIFIED WITH mysql_native_password BY '#{database_data['database_password']}';"
			mysql -u root -proot -e "grant all privileges on \\\`#{database_data['database_name']}\\\`.* to '#{database_data['database_user']}'@'127.0.0.1';"
			mysql -u root -proot -e "grant process on *.* to '#{database_data['database_user']}'@'127.0.0.1';"
			mysql -u root -proot -e "flush privileges;"
		EOF
		timeout 10000
		action :run
	end

	if database_data['dump_hostname']
		# add host to ssh known hosts
		ssh_known_hosts_entry database_data['dump_hostname'] do
			port database_data['dump_port']
		end

		# download the sql data
		bash 'Download Dump for ' + database_data['database_name'] do
			user 'vagrant'
			group 'vagrant'
			timeout 3000000
			code <<-EOF
				rsync --delete -auvz --progress -e "ssh -p #{database_data['dump_port']} -o StrictHostKeyChecking=no" \
					--no-o --no-g #{database_data['dump_user']}@#{database_data['dump_hostname']}:#{database_data['dump_remote']} \
					#{database_data['dump_local']};
			EOF
			action :run
		end
	end

	if database_data['dump_local'] and database_data['database_name']
		# load the dump
		bash 'Import Dump for ' + database_data['database_name'] do
			code <<-EOF
				mysql -u root -proot #{database_data['database_name']} < #{database_data['dump_local']}
			EOF
			timeout 10000
			action :run
		end

		if database_data['post_install_queries']
			database_data['post_install_queries'].each do |query|
				bash 'Execute Query: ' + query do
					code <<-EOF
						mysql -u root -proot #{database_data['database_name']} -e "#{query}"
					EOF
					timeout 10000
					action :run
				end
			end
		end
	end

	if database_data['dump_hostname'] and database_data['dump_local']
		bash 'Remove downloaded ' + database_data['dump_local'] do
			code <<-EOF
				rm -f #{database_data['dump_local']}
			EOF
			action :run
		end
	end
end

##################
### Setup Solr ###
##################

if node['normal_site']['solr']['init']
	openjdk_install '11'
	include_recipe 'typo3_solr'

	typo3_solr_app 'solr' do
		solr node['normal_site']['solr']['version_solr']
		extension node['normal_site']['solr']['version_extension']
		tika_server node['normal_site']['solr']['version_tika_server']
	end

	node['normal_site']['solr']['cores'].each do |core_data|
		typo3_solr_core core_data['name'] do
			language core_data['language']
			config_directory node['normal_site']['solr']['config_directory']
			action :add
		end
	end

	execute 'solr-updateConnections' do
		command "php #{node['normal_site']['webroot']}/#{base_hostname}/#{node['normal_site']['web_directory']}/typo3/cli_dispatch.phpsh extbase solr:updateconnections || true"
		user 'vagrant'
		group 'vagrant'
		action :run
	end
end

############################
### Configure Shopware 6 ###
############################

if node['normal_site']['configure_shopware'] && node['normal_site']['shopware_version'][0] == '6'
	bash 'Configure Shopware 6x if the file "FIRST_INSTALL" exists.' do
		cwd '/home/vagrant/'

		user 'vagrant'
		group 'vagrant'
		timeout 3000000

		code <<-EOF
			if [ -f #{node['normal_site']['webroot']}/#{base_hostname}/FIRST_INSTALL ]
			then
				cd #{node['normal_site']['webroot']}/#{base_hostname}
		
				bin/console system:install --basic-setup --drop-database --create-database -f

				rm -f FIRST_INSTALL
			fi
		EOF
		action :run
	end
end

